package game;

import java.util.*;

/**
 * The state of a game. Frozen and hashable.
 * This is a vertex in the coarse-grained gamestate graph.
 */
public class NormalizedGameState {
	private GameState representativeGameState;
	private HashSet<GameState> connectedComponent;
	private boolean hasWon;
	
	public GameState getRepresentativeGameState() {
		return representativeGameState;
	}
	
	/**
	 * When created, the same representativeGameState must always get the same connectedComponent.
	 */
	public NormalizedGameState(GameState representativeGameState, HashSet<GameState> connectedComponent) {
		this.representativeGameState = representativeGameState;
		this.connectedComponent = new HashSet<GameState>();
		this.connectedComponent.addAll(connectedComponent);
		hasWon=false;
		for (GameState g : connectedComponent) if (g.hasWon()) hasWon=true;
	}
	
	public boolean hasWon() {
		return hasWon;
	}
	
	public Vector<PlayerAction> generatePlayerActions(Long killtime) throws TimeUpException {
		Vector<PlayerAction> ret = new Vector<PlayerAction>();
		for (GameState gs : connectedComponent) {
			Vector<PlayerAction> actions = gs.generatePlayerActions(false);
			for (PlayerAction action : actions) {
				if (!action.getNewGamestate().normalize(killtime).equals(this)) {
					boolean hasANewTarget = true;
					for (PlayerAction testaction : ret) {
						if (testaction.getNewGamestate().normalize(killtime).equals(action.getNewGamestate().normalize(killtime))) {
							hasANewTarget=false;
							if (action.getCost()<testaction.getCost()) {
								ret.set(ret.indexOf(testaction), action);
								break;
							}
						}
					}
					if (hasANewTarget) {
						ret.add(action);
					}
				}
			}
		}
		return ret;
	}

	@Override
	public int hashCode() {
		return representativeGameState.hashCode();
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof NormalizedGameState) {
			NormalizedGameState n = (NormalizedGameState)o;
			return n.representativeGameState.equals(representativeGameState);
		}
		return false;
	}
	
	@Override
	public String toString() {
//		return (hasWon()?"WON! ":"")+connectedComponent.toString();
		return (hasWon()?"WON! ":"")+representativeGameState.toString();
	}
}
