package game;

/**
 * Frozen and hashable.
 */
public class PlayerAction {
	public static boolean simplifiedCosts = false;
	private PlayerActionType type;
	private GameState oldstate;
	private GameState newstate;
	public PlayerAction(PlayerActionType type, GameState oldstate, GameState newstate) {
		if (oldstate==null || newstate==null) throw new Error("states cannot be null");
		this.type = type;
		this.oldstate = oldstate;
		this.newstate = newstate;
	}
	public double getCost() {
		if (simplifiedCosts) return 1;
		return type.getCost();
	}
	public PlayerActionType getType() {
		return type;
	}
	public GameState getNewGamestate() {
		return newstate;
	}
	public GameState getOldGamestate() {
		return oldstate;
	}
	@Override
	public String toString() {
		return type.toString()+"  "+oldstate+"  ->->->  "+newstate;
	}

	public String humanReadable() {
		return "from the state "+oldstate.humanReadable()+" use the action '"+type.toString()+"' to reach the state "+newstate.humanReadable();
	}

	@Override
	public int hashCode() {
		return type.hashCode()*10037+oldstate.hashCode()*1013+newstate.hashCode();
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof PlayerAction) {
			PlayerAction pa = (PlayerAction)o;
			return type==pa.type && oldstate.equals(pa.oldstate) && newstate.equals(pa.newstate);
		}
		return false;
	}
}
